/*
	MOD-RGB Library for the MAPLE IDE
	
	If you have any questions, email
	support@olimex.com
	
	https://www.olimex.com
	OLIMEX 2013

 */	

#ifndef MODRGB_H
#define MODRGB_H

#include "wirish.h"
	
class MODRGB
{
private:
	static MODRGB * currentObj;
	unsigned char pinSDA, pinSCL;
	const static char  ACK = 0;
	const static char NACK = 1;
	const static unsigned char RW_MASK  = 0x80;
	const static unsigned char SL_A     = 0x48;
	const static unsigned char SL_WRITE = 0x05;
	const static unsigned char SL_WRITE2 = 0xA0;
			
public:
	MODRGB( unsigned char nSDA, unsigned char nSCL )
			:pinSDA(nSDA), pinSCL(nSCL){ 
									MODRGB::currentObj = this; 
									SCL_OUT();
									SDA_OUT();
								 }
	~MODRGB(){}
	
	void SDA_OUT(){ pinMode(pinSDA, OUTPUT);}
	void SDA_IN() { pinMode(pinSDA,  INPUT);}
	void SCL_OUT(){ pinMode(pinSCL, OUTPUT);}
	void SCL_IN() { pinMode(pinSCL, INPUT); } 
	
	void Start(){
			SDA_OUT();
			WaitMicrosecond(100);
			SCL_OUT();
			WaitMicrosecond(100);		
	}
		
	void Stop(){
			SDA_OUT();
			WaitMicrosecond(100);
			SCL_IN();
			WaitMicrosecond(100);
			SDA_IN();
			WaitMicrosecond(100);
	}
		
	char WriteSingle( char data ){
			char i;
			
			// Send the 8 bits
			for(i = 0; i<8; i++)
			{
				WaitMicrosecond(100);
				if(data&RW_MASK) SDA_IN(); else SDA_OUT();
				data <<= 1;
				WaitMicrosecond(100);
				SCL_IN();
				WaitMicrosecond(100);
				SCL_OUT();
			}
		
			// Read the ACK
			WaitMicrosecond(100);
			SDA_IN();
			WaitMicrosecond(100);
			SCL_IN();
			WaitMicrosecond(100);
			i = digitalRead(pinSDA);
			SCL_OUT();
			WaitMicrosecond(100);

			return i;		
	}
		
	void SetRGB(unsigned char R, unsigned char G, unsigned char B){		

		Start();
		WriteSingle(0x48<<1);
		WriteSingle(SL_WRITE);
		WriteSingle(SL_WRITE2);
		WriteSingle(0x03);
		WriteSingle(R);
		WriteSingle(G);
		WriteSingle(B);
		Stop();	
	
	}	
	
	void WaitMicrosecond(int x){ delayMicroseconds(x); }	
	
	unsigned char getSDAnum(){ return pinSDA; }
	unsigned char getSCLnum(){ return pinSCL; }
};

MODRGB * MODRGB::currentObj = NULL;

#endif